from django.db.models import Case, When, Q, F, Sum
from django.shortcuts import _get_queryset

from django_pivot.utils import get_column_values


def pivot(queryset, row, column, data, aggregation=Sum, choices='auto'):
    """
    Takes a queryset and pivots it. The result is a table with one record
    per unique value in the `row` column, a column for each unique value in the `column` column
    and values in the table aggregated by the data column.

    :param queryset: a QuerySet, Model, or Manager
    :param row: string, name of column that will key the rows
    :param column: string, name of column that will define columns
    :param data: column name or Combinable
    :param aggregation: aggregation function to apply to data column
    :return: ValuesQueryset
    """
    queryset = _get_queryset(queryset)

    column_values = get_column_values(queryset, column, choices)

    annotations = _get_annotations(column, column_values, data, aggregation)

    return queryset.values(row).annotate(**annotations)


def _get_column_values(queryset, column):
    return queryset.values_list(column, flat=True).distinct()


def _get_annotations(column, column_values, data, aggregation):
    value = data if hasattr(data, 'resolve_expression') else F(data)
    return {
        display_value: aggregation(Case(When(Q(**{column: column_value}), then=value)))
        for column_value, display_value in column_values
    }



from django.utils.encoding import force_text


def get_column_values(queryset, field, choices):
    field_object = _get_field(queryset.model, field)
    if choices == 'auto' and field_object.choices:
        field_values = field_object.choices
    elif choices == 'auto' or choices == 'minimum':
        database_values = queryset.values_list(field, flat=True).distinct().order_by(field)
        field_values = [(field_value, force_text(field_value)) for field_value in database_values]
    else:
        field_values = choices

    return field_values


def _get_field(model, field_names):
    field_list = field_names.split('__')
    for field_name in field_list:
        field = model._meta.get_field(field_name)
        if field.is_relation:
            model = field.related_model
        else:
            return field



from django.db import models


class Region(models.Model):
    name = models.CharField(max_length=7)

    def __unicode__(self):
        return self.name


class Store(models.Model):
    name = models.CharField(max_length=20)
    region = models.ForeignKey(Region)

    def __unicode__(self):
        return self.name


class ShirtSales(models.Model):
    gender = models.CharField(max_length=4)
    style = models.CharField(max_length=5)
    shipped = models.DateField()
    units = models.IntegerField()
    price = models.DecimalField(max_digits=8, decimal_places=2)
    store = models.ForeignKey(Store)
